/* vi:set cindent tabstop=2 shiftwidth=2: */
/*
 * libvxfs - library for reading Veritas Journaled FileSystem (VxFS)
 * Copyright (c) 1999 Martin Hinner <mhi@penguin.cz>
 * Copyright (C) 1992, 93, 94, 95, 96, 97 Free Software Foundation, Inc.
 *
 * scandir.c: vxscandir routine
 *   ( This file is from GNU C Library)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: scandir.c,v 0.3 1999/12/14 19:04:35 mhi Exp root $
 */
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <libvxfs.h>

int
vxalphasort (const void *a, const void *b)
{
	return strcmp ((*(const struct vxdirent **) a)->name,
								 (*(const struct vxdirent **) b)->name);
}


int
vxscandir (struct vxfs *fs,
					 struct vxfs_inode *ino, struct vxdirent ***namelist,
					 int (*select) (struct vxdirent *),
					 int (*cmp) (const void *, const void *))
{
	VXDIR *dp = vxopendir (fs, ino);
	struct vxdirent *d, **v = NULL;
	size_t vsize = 0, i;

	if (dp == NULL)
		return -1;

	i = 0;
	while ((d = vxreaddir (dp)) != NULL)
		if (select == NULL || (*select) (d))
			{
				if (i == vsize)
					{
						struct vxdirent **new;

						if (vsize == 0)
							vsize = 10;
						else
							vsize *= 2;

						new = (struct vxdirent **) realloc (v, vsize * sizeof (*v));
						if (new == NULL)
							{
								break;
							}
						v = new;
					}

				v[i] = (struct vxdirent *) malloc (sizeof (struct vxdirent));
				if (v[i] == NULL)
					break;

				memcpy (v[i++], d, sizeof (struct vxdirent));
			}

	if (errno != 0)
		{
			(void) vxclosedir (dp);
			while (i > 0)
				free (v[--i]);
			free (v);
			return -1;
		}

	(void) vxclosedir (dp);

	/* Sort the list if we have a comparison function to sort with.  */
	if (cmp != NULL)
		qsort (v, i, sizeof (*v), cmp);

	*namelist = v;
	return i;
}
